Rem
Rem $Header: ctre_setup_destination.sql 2015/02/01 11:12:30 dbolton991 Exp $
Rem
Rem ctre_setup_destination.sql
Rem
Rem Copyright (c) 2015, VLSS LLC and/or its affiliates. 
Rem All rights reserved. 
Rem
Rem    NAME
Rem      ctre_setup_destination.sql
Rem
Rem    DESCRIPTION
Rem      SQL*Plus command file to setup Connecteré environment.
Rem
Rem    NOTES
Rem      Must be run from connected to SYS (or internal)
Rem
Rem    MODIFIED   (MM/DD/YY)
Rem

set echo off verify off showmode off feedback off;
whenever sqlerror exit sql.sqlcode

@terms.sql

prompt
prompt Please accept the Terms and Conditions to continue
prompt --------------------------------------------------

prompt The Terms and Conditions must be accepted to continue the
prompt Connecteré install.  Please enter 'Y' or 'Yes' to proceed.
prompt
prompt &&terms_and_conditions

begin
  if UPPER('&&terms_and_conditions') <> 'Y' and UPPER('&&terms_and_conditions') <> 'YES' then
    raise_application_error(-20001, 'Install failed - Terms and Conditions not accepted');
  end if;
end;
/

prompt
prompt Enter the company name provided for licensing
prompt ---------------------------------------------  

prompt
prompt &&company_name

prompt
prompt Enter the license key provided
prompt ------------------------------

prompt
prompt &&license_key

prompt
prompt Choose the Connecteré user
prompt ---------------------------  

prompt Not specifying a user will result in the installation FAILING
prompt
prompt &&ctre_user

prompt
prompt Choose the Connecteré user's password
prompt --------------------------------------  

prompt Not specifying a password will result in the installation FAILING
prompt
prompt &&ctre_password

prompt
prompt Enter the username for the source database user
prompt -----------------------------------------------

prompt This should be the same username that was specified during the
prompt install on the source system
prompt
prompt &&source_user

prompt
prompt Enter the password for the source database user
prompt -----------------------------------------------

prompt This should be the same password that was specified during the
prompt install on the source system
prompt
prompt &&source_password

prompt
prompt Enter the TNS entry for the source system
prompt -----------------------------------------

prompt The TNS entry must be configured and valid, so something like
prompt 'select 2 from dual@<TNS Entry>' should return a result.
prompt
prompt &&source_tns_entry


Rem Begin spooling after all passwords have been entered
spool ctre_setup_destination.lis

begin
  if '&&ctre_user' is null then
    raise_application_error(-20101, 'Install failed - No user specified for Connecteré install');
  end if;
end;
/

begin
  if '&&ctre_password' is null then
    raise_application_error(-20102, 'Install failed - No password specified for Connecteré user');
  end if;
end;
/

begin
  if '&&source_user' is null then
    raise_application_error(-20103, 'Install failed - No source user specified');
  end if;
end;
/

begin
  if '&&source_password' is null then
    raise_application_error(-20104, 'Install failed - No source password specified');
  end if;
end;
/

begin
  if '&&source_tns_entry' is null then
    raise_application_error(-20105, 'Install failed - No TNS entry specified for source system');
  end if;
end;
/

Rem
Rem  Set up Connecteré user's temporary and default tablespaces
Rem

prompt
prompt
prompt Choose the Default tablespace for the &&ctre_user user
prompt ----------------------------------------------------

prompt Below is the list of online tablespaces in this database which can
prompt store user data.  Specifying the SYSTEM tablespace for the user's 
prompt default tablespace will result in the installation FAILING, as 
prompt using SYSTEM for performance data is not supported.
prompt
prompt Choose the &&ctre_user users's default tablespace.  This is the tablespace
prompt in which the CONNECTERE tables and indexes will be created.

column db_default format a28 heading 'CONNECTERE DEFAULT TABLESPACE'
select tablespace_name, contents
     , decode(tablespace_name,'SYSAUX','*') db_default
  from sys.dba_tablespaces 
 where tablespace_name <> 'SYSTEM'
   and contents = 'PERMANENT'
   and status = 'ONLINE'
 order by tablespace_name;

prompt
prompt Pressing <return> will result in CONNECTERE's recommended default
prompt tablespace (identified by *) being used.
prompt

set heading off
col default_tablespace new_value default_tablespace noprint
select 'Using tablespace '||
       upper(nvl('&&default_tablespace','SYSAUX'))||
       ' as &&ctre_user default tablespace.'
     , nvl('&default_tablespace','SYSAUX') default_tablespace
  from sys.dual;
set heading on

begin
  if upper('&&default_tablespace') = 'SYSTEM' then
    raise_application_error(-20106, 'Install failed - SYSTEM tablespace specified for DEFAULT tablespace');
  end if;
end;
/


prompt
prompt
prompt Choose the Temporary tablespace for the &&ctre_user user
prompt --------------------------------------------------------

prompt Below is the list of online tablespaces in this database which can
prompt store temporary data (e.g. for sort workareas).  Specifying the SYSTEM 
prompt tablespace for the user's temporary tablespace will result in the 
prompt installation FAILING, as using SYSTEM for workareas is not supported.

prompt
prompt Choose the &&ctre_user user's Temporary tablespace.

column db_default format a26 heading 'DB DEFAULT TEMP TABLESPACE'
select t.tablespace_name, t.contents
     , decode(dp.property_name,'DEFAULT_TEMP_TABLESPACE','*') db_default
  from sys.dba_tablespaces t
     , sys.database_properties dp
 where t.contents           = 'TEMPORARY'
   and t.status             = 'ONLINE'
   and dp.property_name(+)  = 'DEFAULT_TEMP_TABLESPACE'
   and dp.property_value(+) = t.tablespace_name
 order by tablespace_name;

prompt
prompt Pressing <return> will result in the database's default Temporary 
prompt tablespace (identified by *) being used.
prompt

set heading off
col temporary_tablespace new_value temporary_tablespace noprint
select 'Using tablespace '||
       nvl('&&temporary_tablespace',property_value)||
       ' as &&ctre_user temporary tablespace.'
     , nvl('&&temporary_tablespace',property_value) temporary_tablespace
  from database_properties
 where property_name='DEFAULT_TEMP_TABLESPACE';
set heading on

begin
  if upper('&&temporary_tablespace') = 'SYSTEM' then
    raise_application_error(-20107, 'Install failed - SYSTEM tablespace specified for TEMPORARY tablespace');
  end if;
end;
/

prompt
prompt Enter the file system / ASM location for the datafiles on the destination system
prompt --------------------------------------------------------------------------------

prompt Enter a valid file system location on this system for the datafiles to reside.
prompt
prompt File system example: /u01/app/oracle/oradata
prompt
prompt ASM example: +DATA
prompt
prompt &&datafile_location

begin
  if '&&datafile_location' is null then
    raise_application_error(-20108, 'Install failed - No location specified for datafiles');
  end if;
end;
/

prompt
prompt Enter the shared file system location for staging the files
prompt -----------------------------------------------------------

prompt Enter a valid file system location on the systems for the staging files to reside.
prompt 
prompt This file system must be accessible by both the source and dest systems.
prompt
prompt &&shared_location

begin
  if '&&shared_location' is null then
    raise_application_error(-20110, 'Install failed - No shared location specified for staging files');
  end if;
end;
/

prompt
prompt Enter the type of the source system
prompt -----------------------------------

prompt Enter the value of V$DATABASE.PLATFORM_ID from the source system.
prompt
prompt Hint: 2 = Solaris (64-bit)
prompt       3 = HP-UX (64-bit)
prompt       6 = AIX-Based Systems (64-bit)
prompt
prompt &&platform_id

begin
  if '&&platform_id' is null then
    raise_application_error(-20109, 'Install failed - No PLATFORM_ID specified for source system');
  end if;
end;
/

prompt
prompt
prompt ... Creating &&ctre_user user

create user &&ctre_user 
  identified by &&ctre_password
  default tablespace &&default_tablespace
  temporary tablespace &&temporary_tablespace;

alter user &&ctre_user quota unlimited on &&default_tablespace;

grant create session to &&ctre_user;

grant create database link to &&ctre_user;

create directory ctre_dest_dir as '&&datafile_location';

grant read, write on directory ctre_dest_dir to &&ctre_user;

create directory ctre_shared_dir as '&&shared_location';

grant read, write on directory ctre_shared_dir to &&ctre_user;

grant execute on dbms_file_transfer to &&ctre_user;

grant execute on sys.dbms_backup_restore to &&ctre_user;

grant execute on sys.dbms_crypto to &&ctre_user;

grant READ ANY FILE GROUP, RESUMABLE, EXECUTE ANY PROCEDURE, EXEMPT REDACTION POLICY, EXECUTE ANY TYPE, SELECT ANY TABLE, ADMINISTER SQL MANAGEMENT OBJECT, ADMINISTER RESOURCE MANAGER, BACKUP ANY TABLE, SELECT ANY SEQUENCE, CREATE TABLE to &&ctre_user;

grant exp_full_database, imp_full_database to &&ctre_user;

grant alter tablespace to &&ctre_user;

prompt
prompt
prompt ... Creating user objects

Rem
Rem  Create ctre_user tables / indexes / sequences
Rem

CREATE TABLE &&ctre_user..ctre_parameters (
  name					VARCHAR2(50)  NOT NULL,
  value					VARCHAR2(4000) NOT NULL,
  constraint ctre_parameters_pk		PRIMARY KEY (name));

CREATE TABLE &&ctre_user..ctre_source (
  file#					NUMBER          NOT NULL,
  ts#					NUMBER		NOT NULL,
  name					VARCHAR2(30)    NOT NULL,
  dname					VARCHAR2(4000)	NOT NULL,
  fname					VARCHAR2(4000)  NOT NULL,
  constraint ctre_source_pk		PRIMARY KEY (file#));

CREATE TABLE &&ctre_user..ctre_data_files (
  file#					NUMBER          NOT NULL,
  fname					VARCHAR2(4000)  NOT NULL,
  dname					VARCHAR2(4000)  NOT NULL,
  checkpoint_change#			NUMBER, 
  constraint ctre_data_files_pk		PRIMARY KEY (file#));

CREATE TABLE &&ctre_user..ctre_incrementals (
  file#					NUMBER          NOT NULL,
  fname					VARCHAR2(4000)  NOT NULL,
  checkpoint_change#			NUMBER, 
  is_consistent				VARCHAR2(1),
  is_converted				VARCHAR2(1),
  is_applied				VARCHAR2(1),
  constraint ctre_incrmentals_pk	PRIMARY KEY (file#, fname));

CREATE TABLE &&ctre_user..ctre_exports (
  dumpfile_name				VARCHAR2(100)  NOT NULL,
  is_current				VARCHAR2(1) NOT NULL,
  constraint ctre_exports_pk		PRIMARY KEY (dumpfile_name));

CREATE SEQUENCE &&ctre_user..ctre_seq;

CREATE TABLE &&ctre_user..ctre_logs (
   log_entry_id      NUMBER (10),
   run_number        NUMBER,
   process_name      VARCHAR2 (50),
   description       VARCHAR2 (400),
   start_date        TIMESTAMP,
   end_date	     TIMESTAMP,
   status_code       NUMBER(1),
   run_result        VARCHAR2(200),
   CONSTRAINT ctre_logs_pk 
      PRIMARY KEY (log_entry_id),
   CONSTRAINT ctre_logs_u01
      UNIQUE (run_number, process_name)
);

CREATE SEQUENCE &&ctre_user..ctre_logs_seq;

prompt
prompt
prompt ... Installing required packages

Rem
Rem  Install required packages
Rem

conn &&ctre_user/&&ctre_password;

create database link ctre_link.local connect to &&source_user identified by &&source_password using '&&source_tns_entry';

INSERT INTO ctre_parameters
   VALUES ('stage_location', '&&shared_location');

INSERT INTO ctre_parameters
   VALUES ('dest_location', '&&datafile_location');

INSERT INTO ctre_parameters
   VALUES ('company_name', '&&company_name');

INSERT INTO ctre_parameters
   VALUES ('license_key', NVL('&&license_key', 'NULL'));

INSERT INTO ctre_parameters
   VALUES ('platform_id', NVL('&&platform_id', 'NULL'));

UPDATE ctre_parameters
   SET value = value || '/'
 WHERE SUBSTR(value, LENGTH(value), 1) <> '/'
   AND name LIKE '%location';

prompt
prompt NOTE: The following are errors checks.  If values are returned, please stop the install, 
prompt       fix the errors and try again.
prompt

SELECT t.name "TABLESPACE CONFLICTS!!!"
  FROM v$tablespace@ctre_link.local t, 
       sys.ts$@ctre_link.local ts
 WHERE t.ts# = ts.ts#
   AND decode(ts.contents$, 0, (decode(bitand(ts.flags, 16), 16, 'UNDO',
          'PERMANENT')), 1, 'TEMPORARY') NOT IN ('UNDO','TEMPORARY')
   AND t.name NOT IN ('SYSTEM','SYSAUX')
INTERSECT
SELECT tablespace_name
  FROM dba_tablespaces;

SELECT value "CHARACTERSET DIFFERENCE!!!"
  FROM nls_database_parameters@ctre_link.local 
 WHERE parameter = 'NLS_CHARACTERSET'
MINUS
SELECT value 
  FROM nls_database_parameters 
 WHERE parameter = 'NLS_CHARACTERSET';

SELECT DISTINCT owner "MISSING USERS!!!"
  FROM v$tablespace@ctre_link.local t,
       sys.ts$@ctre_link.local ts,
       dba_tables@ctre_link.local d
 WHERE t.ts# = ts.ts#
   AND decode(ts.contents$, 0, (decode(bitand(ts.flags, 16), 16, 'UNDO',
          'PERMANENT')), 1, 'TEMPORARY') NOT IN ('UNDO','TEMPORARY')
   AND t.name NOT IN ('SYSTEM','SYSAUX')
   AND d.tablespace_name = t.name
MINUS
SELECT username
  FROM dba_users;

@ctre_adm.pks

@ctre_adm.plb

prompt
prompt NOTE:
prompt   Connecteré environment complete. Please check ctre_setup_destination.lis for any errors.
prompt

EXIT;
